// App.cpp : Defines the entry point for the application.
//
#include "stdafx.h"
#include "StickersWindow.h"	
#include "ChatDlg.h"
#include "StickerGroup.h"
#include "NeMessenger.h"
#include "DuiDefine.h"

#define  STICKER_COLUMN_COUNT			4
#define  MAX_RECENT_STICKER_COUNT		12
#define  CURRENT_STICKER_GROUP			0
#define  TABGROUP_ITEM_COUNT			8

//#define kbtnClose			_T("closebtn")
#define kbtnRightArrow		_T("RightArrowBtn") 
#define kbtnLeftArrow		_T("LeftArrowBtn") 
#define kGroupOptions		_T("GroupOptions")

CStickersWindow::CStickersWindow(HWND hwnd)
{
	m_hwndChatDialog =  hwnd;
	m_pStickerList = NULL;
	m_nCurTablGroup = 0;

    //Make sure all stick loaded, 
    //if sticker is loading in CChatSendMsgThread::Run, this will block until load complete,
    //and because will load sticker just once, so this will not cause performance issue
    GET_CONFIG()->GetStickerMgr().Load();	
}

CStickersWindow::~CStickersWindow()
{

}

DUI_BEGIN_MESSAGE_MAP(CStickersWindow, WindowImplBase, FALSE)
    //DUI_ON_MSGTYPE(DUI_MSGTYPE_WINDOWINIT, OnPrepare)
    DUI_ON_CLICK_CTRNAME(kCloseButtonControlName, _OnCtrlCloseWindow)
    DUI_ON_CLICK_CTRNAME(kbtnRightArrow, _OnCtrlRightArrow)
    DUI_ON_CLICK_CTRNAME(kbtnLeftArrow, _OnCtrlLeftArrow)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_CLICK,_OnCtrlSelect)
	
DUI_END_MESSAGE_MAP()

void CStickersWindow::OnFinalMessage( HWND hWnd )
{
	
}

void CStickersWindow::InitWindow() 
{
	SetWindowText(m_hWnd, LoadStringFromResource(IDS_STRING377));

	CButtonUI* pBtnClose =static_cast<CButtonUI*>(m_PaintManager.FindControl(kCloseButtonControlName));
	if (pBtnClose != NULL)
	{
		pBtnClose->SetToolTip(LoadStringFromResource(IDS_STRING_CHAT_TOOLTIP_7));
	}
	CLabelUI* pWindowText =static_cast<CLabelUI*>(m_PaintManager.FindControl(_T("wndTitle")));
	if (pWindowText)
	{
		CString strTitle;
		strTitle.Format(_T("<x 4>%s"),LoadStringFromResource(IDS_STRING377));
		pWindowText->SetText(strTitle);
		pWindowText->SetShowHtml(true);
	}

	m_pStickerList = static_cast<CListUI*>(m_PaintManager.FindControl(_T("stickerlist"))) ;
	_ASSERT(m_pStickerList);

	CHorizontalLayoutUI *phLayout = static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(kGroupOptions)) ;
	if (phLayout)
	{
		for (int i = 0; i < phLayout->GetCount(); i++)
		{
			COptionUI *op = (COptionUI*)phLayout->GetItemAt(i);
			if (op)
			{
				CDuiString strGrpId = op->GetUserData();
				SetGroupImage(_wtoi(strGrpId),FALSE);
			}
		}
	}



	GET_CONFIG()->GetStickerMgr().SetCurrentGroup(CURRENT_STICKER_GROUP);
	SetGroupImage(CURRENT_STICKER_GROUP,TRUE);


	if (0 == GET_NTALKINFO()->strCorp.CompareNoCase(_T("hanatour")))
	{
		if (phLayout)
		{
			for (int i = 1; i < phLayout->GetCount(); i++)
			{
				bool bShow = false;
				COptionUI *op = (COptionUI*)phLayout->GetItemAt(i);
				if (op)
				{
					CDuiString dat = op->GetUserData();
					int v = _wtoi(dat.GetData());
					if (v < 3)
					{
						//Only show group 1,2 for hanatour
						bShow = true;
					}

					op->SetVisible(bShow);
				}

			}
		}

		CButtonUI *pButton = static_cast<CButtonUI*>(m_PaintManager.FindControl(kbtnRightArrow)) ;
		if (pButton)
		{
			pButton->SetEnabled(false);
		}
	}


	ShowCurrentStickerGroup();
}  


CDuiString CStickersWindow::GetSkinFile()
{
	return _T("StickerDlg.xml");
};

CDuiString CStickersWindow::GetSkinFolder()
{
	return _T("skin\\nTalkRes");
};

void CStickersWindow::SetClientRect(CRect rc)
{
	CRect m_rcClient = rc;
	SetWindowPos(GetHWND(), HWND_TOP, m_rcClient.left, m_rcClient.top, m_rcClient.Width(), m_rcClient.Height(), SWP_FRAMECHANGED | SWP_HIDEWINDOW);
	ShowWindow(true, true);

};

bool CStickersWindow::SetCurrentTabGroup(int index)
{
	if (m_nCurTablGroup == index)
	{
		return false;
	}

	CControlUI* pControl = m_PaintManager.FindControl(kGroupOptions);
	if (pControl)
	{
		for (int i = 0; i < TABGROUP_ITEM_COUNT; i++)
		{
			CControlUI *pSubControl = m_PaintManager.FindSubControlByClass(pControl,_T("OptionUI"),m_nCurTablGroup * TABGROUP_ITEM_COUNT + i);
			if (pSubControl)
			{
				pSubControl->SetVisible(false);
			}
		}
		for (int i = 0; i < TABGROUP_ITEM_COUNT; i++)
		{
			CControlUI *pSubControl = m_PaintManager.FindSubControlByClass(pControl,_T("OptionUI"),index * TABGROUP_ITEM_COUNT + i);
			if (pSubControl)
			{
				pSubControl->SetVisible(true);
			}
		}

		m_nCurTablGroup = index;
	}

	return true;
}

void CStickersWindow::_OnCtrlCloseWindow(TNotifyUI& msg)
{
	::ShowWindow(m_hWnd,SW_HIDE);
}

void CStickersWindow::_OnCtrlRightArrow(TNotifyUI& msg)
{
	CHorizontalLayoutUI* pLayout =static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(kGroupOptions));
	if (pLayout)
	{
		int nTabGroupCount = pLayout->GetCount()/TABGROUP_ITEM_COUNT + (pLayout->GetCount() % TABGROUP_ITEM_COUNT == 0?0:1);

		SetCurrentTabGroup(m_nCurTablGroup + 1);

		bool bEnable = m_nCurTablGroup >= nTabGroupCount - 1?false:true;
		msg.pSender->SetEnabled(bEnable);

		CControlUI *pControl = (m_PaintManager.FindControl(kbtnLeftArrow));
		if (pControl)
		{
			pControl->SetEnabled(true);
		}
	}
}

void CStickersWindow::_OnCtrlLeftArrow(TNotifyUI& msg)
{
	CHorizontalLayoutUI* pLayout =static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(kGroupOptions));
	if (pLayout)
	{
		if (m_nCurTablGroup > 0)
		{
			SetCurrentTabGroup(m_nCurTablGroup - 1);
			if (m_nCurTablGroup <= 0)
			{
				msg.pSender->SetEnabled(false);
			}

			CControlUI *pControl = (m_PaintManager.FindControl(kbtnRightArrow));
			if (pControl)
			{
				pControl->SetEnabled(true);
			}
		}
	}
}

void CStickersWindow::_OnCtrlSelect(TNotifyUI& msg)
{
	if (msg.pSender->GetName().Find(_T("StickerItem")) != -1)
	{
		CStickerItem *pStickerItem = (CStickerItem*)msg.pSender->GetTag();
		if (pStickerItem)
		{
			int gid = pStickerItem->GetGroupId();
			int sid = pStickerItem->GetStickerId();
			CDuiString strImgName = pStickerItem->GetImageName();
			if (m_hwndChatDialog != NULL && ::IsWindow(m_hwndChatDialog))
			{
				::SendMessage(m_hwndChatDialog,WM_CHAT_STICKER_SELECTED, (WPARAM)pStickerItem, NULL);
				CStickerMgr &stickerMgr = GET_CONFIG()->GetStickerMgr();

				CStickerGroup *pStickerGroup = stickerMgr.GetRecentStickerGroup();
				if (pStickerGroup->IsExistStickerItem(gid,sid))
				{
					pStickerGroup->Remove(gid,sid);
				}

				if (pStickerGroup->GetStickerCount() >= MAX_RECENT_STICKER_COUNT)
				{
					CStickerItem *psi = pStickerGroup->GetStickerItem(pStickerGroup->GetStickerCount() - 1);
					pStickerGroup->Remove(psi->GetGroupId(),psi->GetStickerId());
				}

				pStickerGroup->AddFront(new CStickerItem(gid,sid));
				stickerMgr.Save();

				ShowCurrentStickerGroup();

			}
		}
	}
	else if (msg.pSender->GetName().Find(_T("StickerGroup")) != -1)
	{
		int grpid = _wtoi(msg.pSender->GetUserData()); //the group id of sticker

		CStickerGroup *psg = GET_CONFIG()->GetStickerMgr().GetCurrentGroup();
		if (psg)
		{
			SetGroupImage(psg->GetGroupId(),FALSE);
		}

		GET_CONFIG()->GetStickerMgr().SetCurrentGroup(grpid);
		SetGroupImage(grpid,TRUE);
		ShowCurrentStickerGroup();
	}
}

void CStickersWindow::Reset()
{
	SetCurrentTabGroup(0);
	CButtonUI* pBtn =static_cast<CButtonUI*>(m_PaintManager.FindControl(kbtnLeftArrow));
	if (pBtn)
	{
		pBtn->SetEnabled(false);
	}

	pBtn =static_cast<CButtonUI*>(m_PaintManager.FindControl(kbtnRightArrow));
	if (pBtn)
	{
		pBtn->SetEnabled(true);
	}

	CStickerGroup *psg = GET_CONFIG()->GetStickerMgr().GetCurrentGroup();
	if (psg)
	{
		SetGroupImage(psg->GetGroupId(),FALSE);
	}

	GET_CONFIG()->GetStickerMgr().SetCurrentGroup(0);
	SetGroupImage(0,TRUE);
	ShowCurrentStickerGroup();
}

void CStickersWindow::SetGroupImage(int nGroupId,BOOL bSelected)
{
	CDuiString strGroupName;
	strGroupName.Format(_T("StickerGroup%d"),nGroupId);

	COptionUI* pOptionUI =static_cast<COptionUI*>(m_PaintManager.FindControl(strGroupName));
	if (pOptionUI)
	{
		CDuiString strImage = GetAppImgDir() + _T("\\sticker\\tab\\btn_chat_talk_icon_0");
		strImage += '0' + nGroupId;
		strImage += bSelected?_T("_pre.png"):_T("_nor.png");

		pOptionUI->SetForeImage(strImage);
	}
}


int CStickersWindow::ShowCurrentStickerGroup()
{
	CStickerMgr &stickerMgr = GET_CONFIG()->GetStickerMgr();

	m_pStickerList->RemoveAll();

	CStickerGroup *pStickerGroup = stickerMgr.GetCurrentGroup();
	if (!pStickerGroup)
	{
		return -1;
	}

	CDuiString strSGName;
	strSGName.Format(_T("StickerGroup%d"),pStickerGroup->GetGroupId());
	COptionUI* pOptionUI =static_cast<COptionUI*>(m_PaintManager.FindControl(strSGName));
	if (pOptionUI)
	{
		pOptionUI->Selected(TRUE);
	}

	//CLabelUI* pWindowText =static_cast<CLabelUI*>(m_PaintManager.FindControl(_T("wndTitle")));

	CHorizontalLayoutUI *pLayout = NULL;
	for (int i = 0; i < pStickerGroup->GetStickerCount(); i++)
	{
		CStickerItem *pStickerItem = pStickerGroup->GetStickerItem(i);
		if (i % STICKER_COLUMN_COUNT == 0)
		{
			CListContainerElementUI *plce = new CListContainerElementUI();
			pLayout = new CHorizontalLayoutUI();
			if (!plce || !pLayout)
			{
				SAFE_DELETE(plce);
				SAFE_DELETE(pLayout);
				return -1;
			}

			m_pStickerList->Add(plce);
			plce->Add(pLayout);

			RECT rcPadding = {10,7,9,0};
			plce->SetPadding(rcPadding);
			plce->SetFixedHeight(78);		

		}


		if (pLayout)
		{	
			COptionUI *option = new COptionUI();
			if (!option)
			{
				return -1;
			}			

			pLayout->Add(option);

			CDuiString strStickerName;
			strStickerName.Format(_T("StickerItem_%ud"),(ULONG)pStickerItem);
			option->SetName(strStickerName);
			option->SetFixedWidth(70);

			RECT rcPadding = {7,0,0,0};
			option->SetPadding(rcPadding);
			

			option->SetGroup(_T("stickerGroup"));
			option->SetTag((UINT_PTR)pStickerItem);

			CString strImageName = pStickerItem->GetImageName();
			option->SetHotImage(_T("file='btn_chat_talk_sticker_bg_sel_03.png'"));
			option->SetSelectedImage(_T("file='btn_chat_talk_sticker_bg_sel_03.png'"));
			option->SetForeImage(strImageName);
		}
	}
	return 0;
}

LRESULT CStickersWindow::OnKillFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	::ShowWindow(m_hWnd,SW_HIDE);
	return 0;
}
LRESULT CStickersWindow::OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	bHandled = FALSE;
	if (wParam == VK_ESCAPE)
	{
		bHandled = TRUE;
		::ShowWindow(m_hWnd,SW_HIDE);
	}
	return 0;
}
